require_relative '../spec_helper'
require_relative '../../../libraries/helpers'

describe Manage::Helpers do
  subject(:helpers) { Object.new.extend(described_class) }

  describe 'chef_server_external_url' do
    subject(:chef_server_external_url) { helpers.chef_server_external_url(node) }

    let(:node) { { 'private_chef' => { 'lb' => { 'api_fqdn' => 'example.com' },
                                       'nginx' => {} }, } }

    context 'when the private_chef.nginx.ssl_port is 443' do
      before :each do
        node['private_chef']['nginx']['ssl_port'] = 443
      end

      it 'does not include the port number' do
        expect(chef_server_external_url).to eq 'https://example.com'
      end
    end

    context 'when the private_chef.nginx.ssl_port is not 443' do
      before :each do
        node['private_chef']['nginx']['ssl_port'] = 4443
      end

      it 'includes the port number' do
        expect(chef_server_external_url).to eq 'https://example.com:4443'
      end
    end
  end

  describe 'chef_server_internal_url' do
    subject(:chef_server_internal_url) { helpers.chef_server_internal_url(node) }

    let(:node) { { 'private_chef' => { 'nginx' => {} }, } }

    context 'when the private_chef.nginx.ssl_port is 443' do
      before :each do
        node['private_chef']['nginx']['ssl_port'] = 443
      end

      it 'does not include the port number' do
        expect(chef_server_internal_url).to eq "https://localhost"
      end
    end

    context 'when the private_chef.nginx.ssl_port is not 443' do
      before :each do
        node['private_chef']['nginx']['ssl_port'] = 4443
      end

      it 'includes the port number' do
        expect(chef_server_internal_url).to eq "https://localhost:4443"
      end
    end
  end


  describe 'use_standalone_nginx_server?' do
    subject(:use_standalone_nginx_server?) { helpers.use_standalone_nginx_server?(node) }

    let(:node) do
      {
        'chef-manage' => {
          'fqdn'        => 'example.com',
          'public_port' => 1234,
        },
        'private_chef' => {
          'lb'    => { 'api_fqdn' => 'example.com' },
          'nginx' => { 'ssl_port' => 1234 },
        },
      }
    end

    context 'when using the same fqdn and port as the api' do
      it 'is false' do
        expect(use_standalone_nginx_server?).to eq false
      end
    end

    context 'when using the same fqdn and different port as the api' do
      before :each do
        node['chef-manage']['public_port'] = 4321
      end

      it 'is true' do
        expect(use_standalone_nginx_server?).to eq true
      end
    end

    context 'when a diffrent fqdn and the same port as the api' do
      before :each do
        node['chef-manage']['fqdn'] = 'example2.com'
      end

      it 'is true' do
        expect(use_standalone_nginx_server?).to eq true
      end
    end

    context 'when a diffrent fqdn and different port than the api' do
      before :each do
        node['chef-manage']['fqdn'] = 'example2.com'
        node['chef-manage']['public_port'] = 4321
      end

      it 'is true' do
        expect(use_standalone_nginx_server?).to eq true
      end
    end
  end
end
