require_relative '../spec_helper'

describe 'omnibus-chef-manage::config' do
  let(:chef_run) do
    ChefSpec::SoloRunner.new do |node|
      set_default_attributes_for(node)
    end.converge(described_recipe)
  end

  it 'renders settings.yml' do
    expect(chef_run).to render_file('/var/opt/chef-manage/etc/settings.yml')
  end

  describe 'chef_server_external_url attribute' do
    it 'uses the api_fqdn prefixed with https' do
      expect(chef_run.node['chef-manage']['chef_server_external_url']).to eq(
        'https://example.com'
      )
    end
  end

  describe 'chef_server_internal_url attribute' do
    it 'uses the localhost prefixed with https' do
      expect(chef_run.node['chef-manage']['chef_server_internal_url']).to eq 'https://localhost'
    end
  end

  describe 'fqdn attribute' do
    it 'uses the hostname of the private_chef.lb.api_fqdn' do
      expect(chef_run.node['chef-manage']['fqdn']).to eq 'example.com'
    end
  end

  describe 'origin attribute' do
    it 'uses the hostname the fqdn attribute' do
      expect(chef_run.node['chef-manage']['origin']).to eq(
        chef_run.node['chef-manage']['fqdn']
      )
    end
  end

  describe 'platform.public_url attribute' do
    it 'uses the chef_server_external_url attribute' do
      expect(chef_run.node['chef-manage']['platform']['public_url']).to eq(
        chef_run.node['chef-manage']['chef_server_external_url']
      )
    end
  end

  describe 'platform.url attribute' do
    it 'uses the chef_server_internal_url attribute' do
      expect(chef_run.node['chef-manage']['platform']['url']).to eq(
        chef_run.node['chef-manage']['chef_server_internal_url']
      )
    end
  end
end
