require_relative '../spec_helper'

describe 'omnibus-chef-manage::nginx' do
  let(:chef_run) do
    ChefSpec::SoloRunner.new do |node|
      set_default_attributes_for(node)

      node.set['chef-manage']['fqdn'] = 'example1.com'
      node.set['private_chef']['lb']['api_fqdn'] = 'example2.com'
      node.set['private_chef']['nginx']['ssl_protocols'] = "TLS or GTFO"
      node.set['private_chef']['nginx']['ssl_ciphers'] = "ONLY!THE!GOOD!ONES"
    end.converge(described_recipe)
  end

  context 'using a standalone configuration' do
    it 'uses the private_chef SSL ciphers' do
      expect(chef_run).to render_file('/var/opt/opscode/nginx/etc/nginx.d/manage.conf')
        .with_content("ssl_ciphers ONLY!THE!GOOD!ONES;")
    end

    it 'uses the private_chef SSL protocols' do
      expect(chef_run).to render_file('/var/opt/opscode/nginx/etc/nginx.d/manage.conf')
        .with_content("ssl_protocols TLS or GTFO;")
    end
  end

  %w[ upstreams internal external ].each do |file|
    it "deletes the old opscode-manage #{file} config" do
      expect(chef_run).to delete_file("/var/opt/opscode/nginx/etc/addon.d/30-opscode-manage_#{file}.conf")
    end
  end
end
