class DataBagItemsController < ChefSolrSearchableObjectController
  def create
    databag_item = Chef::DataBagItem.json_create({
      'raw_data' => params[:data].merge({ 'id' => params[:id] }),
      'data_bag' => params[:data_bag_id]
    })

    server.post("data/#{params[:data_bag_id]}", databag_item)
    render json: databag_item.to_hash, status: :created
  end

  def update
    databag_item = Chef::DataBagItem.json_create({
      'raw_data' => params[:data].merge({ 'id' => params[:id] }),
      'data_bag' => params[:data_bag_id]
    })
    server.put("data/#{params[:data_bag_id]}/#{params[:id]}", databag_item)
    render json: databag_item.to_hash, status: :created
  end

  def destroy
    render json: server.delete("data/#{params[:data_bag_id]}/#{params[:id]}")
  end

  protected

  def has_object_permission(obj)
    server.get_rest(show_url(obj.id))
  end

  def object_for_id(id)
    Chef::DataBagItem.json_create({
      'data_bag' => params[:data_bag_id],
      'raw_data' => server.get_rest(show_url(id))
    })
  end

  def object_type
    params[:data_bag_id]
  end

  def searchable_fields
    ['id']
  end

  def index_url
    "data/#{params[:data_bag_id]}"
  end

  # Bag items don't have their own ACLs
  def object_matched_aces(obj)
    fetch_matched_aces("data/#{params[:data_bag_id]}")
  end

  def object_to_hash(obj)
    unless obj.is_a?(Hash)
      obj = obj.raw_data
      # Remove some keys injected by the search layer. HULK SMASH.
      obj.delete('chef_type')
      obj.delete('data_bag')
    end
    {data: obj, access: object_matched_aces(obj), id: obj['id'], bag: params[:data_bag_id]}
  end

  def preload_collection
    bag_id = params[:data_bag_id].to_json
    "databags.add({name: #{bag_id}}).get(#{bag_id}).items; org.databags.currentPage = -1"
  end

  def preload_skip_page
    true
  end
end
