# Worker tasks for Nodes

require_dependency 'util_worker.rb'

module NodesWorkers

  class ManageTagsWorker < BaseWorker
    task_label 'Manage tags'
    def perform(nodes, tags, operation)
      progress(0, nodes.size)
      tags = tags.split(",")
      nodes.each_with_index do |node_id, i|
        node = server.get_rest("nodes/#{node_id}")
        node.set['tags'] = [] unless node.set['tags'].is_a?(Array)
        current_node_tags = node.set['tags']
        case operation
        when 'Add'
          node.set['tags'] = (node.set['tags'] + tags).uniq
        when 'Delete'
          node.set['tags'] -= tags
        when 'Set'
          node.set['tags'] = tags
        end
        if current_node_tags != node.set['tags']
          server.put_rest("nodes/#{node_id}", node)
          expire(:nodes, node_id)
        end
        progress(i+1)
      end
    end
  end

  class EditRunListWorker < UtilWorker::EditRunListWorker
    def expire_run_list(object_id)
      expire(:nodes, object_id)
    end

    def chef_url(object_id)
      "nodes/#{object_id}"
    end
  end

  class NormalWorker < ::AttributesWorker
    task_label 'Update node attributes'
    def chef_url(node_id)
      "nodes/#{node_id}"
    end

    def update_attributes(node, attributes)
      node.normal_attrs = attributes
    end

    def expire_attributes(node_id)
      expire(:nodes, node_id)
    end
  end

end
