require 'uri'
require 'raven_settings'

Chef::Manage::Application.configure do
  rails_config = config
  SecureHeaders::Configuration.configure do |config|
    config.x_frame_options = 'DENY'
    default = ['self']

    if rails_config.action_controller.asset_host.present?
      asset_uri = URI(rails_config.action_controller.asset_host)
      default << "#{asset_uri.scheme}://#{asset_uri.host}:#{asset_uri.port}"
    end

    sentry_src = if RavenSettings.public_dsn.present?
      sentry_uri = URI(RavenSettings.public_dsn)
      ["#{sentry_uri.scheme}://#{sentry_uri.host}:#{sentry_uri.port}"]
    else
      []
    end

    google_analytics_src = ['https://ssl.google-analytics.com']
    marketo_src = ['https://munchkin.marketo.net',
                   'http://munchkin-cdn.marketo.net',
                   'https://munchkin-cdn.marketo.net',
                   "http://#{Settings.marketo.try(:account_id)}.mktoresp.com",
                   "https://#{Settings.marketo.try(:account_id)}.mktoresp.com"]
    demandbase_src = [
      'http://scripts.demandbase.com',
      'https://scripts.demandbase.com',
      'http://api.demandbase.com',
      'https://api.demandbase.com',
      'http://b.company-target.com',
      'https://b.company-target.com'
    ]

    csp = {
      :default_src => default,
      :font_src => default + ['themes.googleusercontent.com'],
      :style_src => default + ['inline', 'fonts.googleapis.com'],
      :script_src => default + google_analytics_src + marketo_src + demandbase_src +
        ['inline', 'eval'],
      :script_nonce => lambda { |controller| controller.params[:script_nonce] = SecureRandom.hex(16) },
      :img_src => default + sentry_src + google_analytics_src + marketo_src +
        demandbase_src + ['blob:'],
      :object_src => ['none'],
      :media_src => ['none'],
      :frame_src => default + demandbase_src,
      :connect_src => default + marketo_src + demandbase_src,
    }

    if Rails.env.production?
      csp[:enforce] = true
    else
      csp[:enforce] = false
      csp[:report_uri] = 'csp_reports'
    end

    config.csp = csp
  end
end
