# Helpers for setting up a Chef::REST connection
require 'chef/rest'
require 'veil'

module ChefRestHelpers

  include SecretsHelpers
  Chef::REST::RESTRequest.user_agent = "Chef Manage#{Chef::REST::RESTRequest::UA_COMMON}"

  # Takes a Net::HTTPServerException, usually returned from the Chef server,
  # and attempts to parse the body as JSON. If that doesn't work, it attempts
  # to give a more generic error
  def parsed_json_for_error(error)
    JSON.parse(error.response.body)
  rescue JSON::ParserError
    { 'error' => error.message }
  end

  protected

  def server(user, organization)
    raise "Username and organization are required" unless user.present? && organization.present?
    _server(user, "#{platform_config.url}/organizations/#{organization}")
  end

  def server_root
    @server_root ||= _server(platform_config.user, platform_config.url)
  end

  # User the root url as a specified user
  def server_user_root(user)
    _server(user, platform_config.url)
  end

  def _server(user, url)
    raise ArgumentError.new "A privileged key is not available on this system. Ensure that you have run 'chef-server-ctl reconfigure' before continuing." unless defined? KEY
    Chef::REST.new(url, user, nil, :headers => chef_rest_helper_headers,
                                   :raw_key => KEY)
  end

  private

  def chef_rest_helper_headers
    @chef_rest_helper_headers ||= { 'X-Ops-Request-Source' => 'web' }.merge(
      platform_config.webkey_tag ? { 'X-Ops-Webkey-Tag' => platform_config.webkey_tag }
                                 : {})
  end

  def platform_config
    @platform_config ||= Settings.platform
  end
end
