# Marketo methods to be used in the sessions controller
module Marketo
  def submit_user_to_marketo
    lead_attributes = {
      FirstName: params[:user][:first_name],
      LastName: params[:user][:last_name],
      Email: params[:user][:email],
      Title: params[:user][:job_title],
      Company: params[:user][:company],
      Phone: params[:user][:phone],
      Describe_Your_Current_Infrastructure__c: params[:comment],
      'opscode-username' => params[:user][:username],
      'product-interest' => 'Enterprise Chef',
      formname: params[:ref],
    }

    if Settings.demandbase
      AccountsHelper.demandbase_field_names.each do |name|
        lead_attributes[name.to_sym] = params[name]
      end
    end

    # Special case because it is a bool
    terms_attr = {attr_name: 'TOU_Accepted', attr_type: 'boolean', attr_value: params[:user][:terms].present? ? '1' : '0'}
    lead_record = {
      email: params[:user][:email],
      foreign_sys_person_id: params[:user][:username],
      foreign_sys_type: 'CUSTOM',
      lead_attribute_list: {
        attribute: lead_attributes.inject([terms_attr]) do |memo, (attr, val)|
          if val.present?
            memo << {attr_name: attr.to_s, attr_type: 'string', attr_value: val}
          end
          memo
        end
      },
    }
    sync_lead_params = {lead_record: lead_record}
    sync_lead_params[:marketo_cookie] = cookies['_mkto_trk'] if cookies['_mkto_trk'].present?
    sync_lead(sync_lead_params)
  end

  def sync_org_with_marketo(username, orgname)
    if cookies['_mkto_trk'].present?
      sync_lead(
        lead_record: {
          foreign_sys_person_id: username,
          foreign_sys_type: 'CUSTOM',
          lead_attribute_list: {
            attribute: [
              { attr_name: 'OHC_Org_Name__c',
                attr_type: 'string',
                attr_value: orgname
              }
            ]
          }
        },
        marketo_cookie: cookies['_mkto_trk']
      )
    end
  end

  # Sync this user's Munchkin profile with their main lead info
  def sync_user_with_marketo(user)
    if cookies['_mkto_trk'].present?
      sync_lead(
        lead_record: {
          foreign_sys_person_id: params[:username],
          foreign_sys_type: 'CUSTOM',
          lead_attribute_list: {
            attribute: [
              { attr_name: 'opscode-username', attr_type: 'string', attr_value: user['username'] },
              { attr_name: 'Email', attr_type: 'string', attr_value: user['email'] }
            ],
          },
        },
        marketo_cookie: cookies['_mkto_trk']
      )
    end
  end

  def marketo_client
    config = Settings.marketo
    return unless config.id && config[:key] # Hash#key is a built-in
    now = Time.now.utc.iso8601
    Savon.client do
      wsdl 'http://app.marketo.com/soap/mktows/2_0?WSDL'
      endpoint config.endpoint if config.endpoint
      soap_header 'tns:AuthenticationHeader' => {
        "mktowsUserId" => config.id,
        "requestSignature" => OpenSSL::HMAC.hexdigest('sha1', config[:key], now + config.id),
        "requestTimestamp" => now,
      }
      if Rails.env.development?
        log_level :debug
        pretty_print_xml true
      end
    end
  end

  def sync_lead(message)
    return unless marketo_client
    begin
      marketo_client.call(:sync_lead, message: message)
    rescue Errno::ECONNRESET, Errno::ETIMEDOUT, Savon::SOAPFault, Timeout::Error => e
      raise if Rails.env.development?
      Rails.logger.error e.message
      Rails.logger.error e.backtrace.join("\n") if e.backtrace.present?
    end
  end
end
